package invite_r

import (
	"git.hilo.cn/hilo-common/domain"
	"git.hilo.cn/hilo-common/mycontext"
	"github.com/gin-gonic/gin"
	"hilo-user/domain/model/invite_m"
	"hilo-user/domain/model/promotion_m"
	"hilo-user/domain/model/user_m"
	"hilo-user/myerr/bizerr"
	"hilo-user/resp"
)

// @Tags 新人邀请
// @Summary 提交申请
// @Param newUserCode formData string true "被邀请人id"
// @Param platform formData string true "平台"
// @Param recharge formData string true "充值金额"
// @Param userCode formData string true "邀请人id"
// @Param videoUrl formData string true "充值金额"
// @Success 200
// @Router /v2/user/invite/apply [post]
func InviteApply(c *gin.Context) (*mycontext.MyContext, error) {
	myCtx := mycontext.CreateMyContext(c.Keys)

	type paramStr struct {
		NewUserCode string `form:"newUserCode" binding:"required"`
		Platform    string `form:"platform" binding:"required"`
		Recharge    string `form:"recharge" binding:"required"`
		UserCode    string `form:"userCode" binding:"required"`
		VideoUrl    string `form:"videoUrl" binding:"required"`
	}

	var param paramStr
	if err := c.ShouldBind(&param); err != nil {
		return myCtx, err
	}
	model := domain.CreateModelContext(myCtx)
	// 平台是否填写正确
	platforms := promotion_m.GetPromotionPlatforms(model)
	var existsPlatform bool
	for _, v := range platforms {
		if v == param.Platform {
			existsPlatform = true
		}
	}
	if !existsPlatform {
		model.Log.Errorf("InviteApply param:%v", param)
		return myCtx, bizerr.InvalidParameter
	}
	// code 是否存在
	newUser, err := user_m.GetUserByCode(model, param.NewUserCode)
	if err != nil {
		model.Log.Errorf("InviteApply param:%v", param)
		return myCtx, err
	}
	user, err := user_m.GetUserByCode(model, param.UserCode)
	if err != nil {
		model.Log.Errorf("InviteApply param:%v", param)
		return myCtx, err
	}
	if newUser.ID == 0 || user.ID == 0 {
		model.Log.Errorf("InviteApply param:%v", param)
		return myCtx, bizerr.InvalidParameter
	}
	// 邀请人是否有资格邀请
	if !promotion_m.IsPromotionAgent(model, user.ID) {
		model.Log.Errorf("InviteApply 没有邀请资格 param:%v", param)
		return myCtx, bizerr.InviteApplyNoPermission
	}
	// 被邀请人是否符合条件
	isInvite, err := promotion_m.IsPromotionInvitee(model, user.ID)
	if err != nil {
		model.Log.Errorf("InviteApply param:%v", param)
		return myCtx, err
	}
	if isInvite {
		model.Log.Errorf("InviteApply 已经被别人邀请了 param:%v", param)
		return myCtx, bizerr.InviteApplyAlreadyInvited
	}
	// 插入邀请表
	err = invite_m.CreateInviteApply(model, user.ID, newUser.ID, param.Platform, param.Recharge, param.VideoUrl)
	if err != nil {
		model.Log.Errorf("InviteApply param:%v", param)
		return myCtx, err
	}

	resp.ResponseOk(c, nil)
	return myCtx, nil
}
